@@ -3,18 +3,17 @@
  * joinpath.c
  *	  Routines to find all possible paths for processing a set of joins
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/joinpath.c,v 1.67 2001/11/11 19:18:54 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/path/joinpath.c,v 1.91.4.3 2005/10/25 20:30:35 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
-#include <sys/types.h>
 #include <math.h>
 
 #include "optimizer/clauses.h"
@@ -24,6 +23,7 @@
 #include "parser/parsetree.h"
 #include "utils/lsyscache.h"
 
+
 static void sort_inner_and_outer(Query *root, RelOptInfo *joinrel,
 					 RelOptInfo *outerrel, RelOptInfo *innerrel,
 					 List *restrictlist, List *mergeclause_list,
@@ -32,18 +32,9 @@ static void match_unsorted_outer(Query *root, RelOptInfo *joinrel,
 					 RelOptInfo *outerrel, RelOptInfo *innerrel,
 					 List *restrictlist, List *mergeclause_list,
 					 JoinType jointype);
-
-#ifdef NOT_USED
-static void match_unsorted_inner(Query *root, RelOptInfo *joinrel,
-					 RelOptInfo *outerrel, RelOptInfo *innerrel,
-					 List *restrictlist, List *mergeclause_list,
-					 JoinType jointype);
-#endif
 static void hash_inner_and_outer(Query *root, RelOptInfo *joinrel,
 					 RelOptInfo *outerrel, RelOptInfo *innerrel,
 					 List *restrictlist, JoinType jointype);
-static Path *best_innerjoin(List *join_paths, List *outer_relid,
-			   JoinType jointype);
 static List *select_mergejoin_clauses(RelOptInfo *joinrel,
 						 RelOptInfo *outerrel,
 						 RelOptInfo *innerrel,
@@ -151,8 +142,10 @@ sort_inner_and_outer(Query *root,
 					 JoinType jointype)
 {
 	bool		useallclauses;
+	Path	   *outer_path;
+	Path	   *inner_path;
 	List	   *all_pathkeys;
-	List	   *i;
+	ListCell   *l;
 
 	/*
 	 * If we are doing a right or full join, we must use *all* the
@@ -162,6 +155,9 @@ sort_inner_and_outer(Query *root,
 	{
 		case JOIN_INNER:
 		case JOIN_LEFT:
+		case JOIN_IN:
+		case JOIN_UNIQUE_OUTER:
+		case JOIN_UNIQUE_INNER:
 			useallclauses = false;
 			break;
 		case JOIN_RIGHT:
@@ -169,12 +165,34 @@ sort_inner_and_outer(Query *root,
 			useallclauses = true;
 			break;
 		default:
-			elog(ERROR, "sort_inner_and_outer: unexpected join type %d",
+			elog(ERROR, "unrecognized join type: %d",
 				 (int) jointype);
-			useallclauses = false;	/* keep compiler quiet */
+			useallclauses = false;		/* keep compiler quiet */
 			break;
 	}
 
+	/*
+	 * We only consider the cheapest-total-cost input paths, since we are
+	 * assuming here that a sort is required.  We will consider
+	 * cheapest-startup-cost input paths later, and only if they don't
+	 * need a sort.
+	 *
+	 * If unique-ification is requested, do it and then handle as a plain
+	 * inner join.
+	 */
+	outer_path = outerrel->cheapest_total_path;
+	inner_path = innerrel->cheapest_total_path;
+	if (jointype == JOIN_UNIQUE_OUTER)
+	{
+		outer_path = (Path *) create_unique_path(root, outerrel, outer_path);
+		jointype = JOIN_INNER;
+	}
+	else if (jointype == JOIN_UNIQUE_INNER)
+	{
+		inner_path = (Path *) create_unique_path(root, innerrel, inner_path);
+		jointype = JOIN_INNER;
+	}
+
 	/*
 	 * Each possible ordering of the available mergejoin clauses will
 	 * generate a differently-sorted result path at essentially the same
@@ -206,9 +224,9 @@ sort_inner_and_outer(Query *root,
 												  mergeclause_list,
 												  outerrel);
 
-	foreach(i, all_pathkeys)
+	foreach(l, all_pathkeys)
 	{
-		List	   *front_pathkey = lfirst(i);
+		List	   *front_pathkey = (List *) lfirst(l);
 		List	   *cur_pathkeys;
 		List	   *cur_mergeclauses;
 		List	   *outerkeys;
@@ -216,10 +234,10 @@ sort_inner_and_outer(Query *root,
 		List	   *merge_pathkeys;
 
 		/* Make a pathkey list with this guy first. */
-		if (i != all_pathkeys)
+		if (l != list_head(all_pathkeys))
 			cur_pathkeys = lcons(front_pathkey,
-								 lremove(front_pathkey,
-										 listCopy(all_pathkeys)));
+								 list_delete_ptr(list_copy(all_pathkeys),
+												 front_pathkey));
 		else
 			cur_pathkeys = all_pathkeys;		/* no work at first one... */
 
@@ -236,7 +254,7 @@ sort_inner_and_outer(Query *root,
 
 		/* Forget it if can't use all the clauses in right/full join */
 		if (useallclauses &&
-			length(cur_mergeclauses) != length(mergeclause_list))
+		  list_length(cur_mergeclauses) != list_length(mergeclause_list))
 			continue;
 
 		/*
@@ -253,20 +271,18 @@ sort_inner_and_outer(Query *root,
 												   cur_mergeclauses,
 												   innerrel);
 		/* Build pathkeys representing output sort order. */
-		merge_pathkeys = build_join_pathkeys(root, joinrel, outerkeys);
+		merge_pathkeys = build_join_pathkeys(root, joinrel, jointype,
+											 outerkeys);
 
 		/*
-		 * And now we can make the path.  We only consider the cheapest-
-		 * total-cost input paths, since we are assuming here that a sort
-		 * is required.  We will consider cheapest-startup-cost input
-		 * paths later, and only if they don't need a sort.
+		 * And now we can make the path.
 		 */
 		add_path(joinrel, (Path *)
 				 create_mergejoin_path(root,
 									   joinrel,
 									   jointype,
-									   outerrel->cheapest_total_path,
-									   innerrel->cheapest_total_path,
+									   outer_path,
+									   inner_path,
 									   restrictlist,
 									   merge_pathkeys,
 									   cur_mergeclauses,
@@ -283,8 +299,9 @@ sort_inner_and_outer(Query *root,
  *	  only outer paths that are already ordered well enough for merging).
  *
  * We always generate a nestloop path for each available outer path.
- * In fact we may generate as many as three: one on the cheapest-total-cost
- * inner path, one on the cheapest-startup-cost inner path (if different),
+ * In fact we may generate as many as four: one on the cheapest-total-cost
+ * inner path, one on the same with materialization, one on the
+ * cheapest-startup-cost inner path (if different),
  * and one on the best inner-indexscan path (if any).
  *
  * We also consider mergejoins if mergejoin clauses are available.	We have
@@ -315,22 +332,29 @@ match_unsorted_outer(Query *root,
 					 List *mergeclause_list,
 					 JoinType jointype)
 {
+	JoinType	save_jointype = jointype;
 	bool		nestjoinOK;
 	bool		useallclauses;
-	Path	   *bestinnerjoin;
-	List	   *i;
+	Path	   *inner_cheapest_startup = innerrel->cheapest_startup_path;
+	Path	   *inner_cheapest_total = innerrel->cheapest_total_path;
+	Path	   *matpath = NULL;
+	Path	   *bestinnerjoin = NULL;
+	ListCell   *l;
 
 	/*
-	 * Nestloop only supports inner and left joins.  Also, if we are doing
-	 * a right or full join, we must use *all* the mergeclauses as join
-	 * clauses, else we will not have a valid plan.  (Although these two
-	 * flags are currently inverses, keep them separate for clarity and
-	 * possible future changes.)
+	 * Nestloop only supports inner, left, and IN joins.  Also, if we are
+	 * doing a right or full join, we must use *all* the mergeclauses as
+	 * join clauses, else we will not have a valid plan.  (Although these
+	 * two flags are currently inverses, keep them separate for clarity
+	 * and possible future changes.)
 	 */
 	switch (jointype)
 	{
 		case JOIN_INNER:
 		case JOIN_LEFT:
+		case JOIN_IN:
+		case JOIN_UNIQUE_OUTER:
+		case JOIN_UNIQUE_INNER:
 			nestjoinOK = true;
 			useallclauses = false;
 			break;
@@ -340,7 +364,7 @@ match_unsorted_outer(Query *root,
 			useallclauses = true;
 			break;
 		default:
-			elog(ERROR, "match_unsorted_outer: unexpected join type %d",
+			elog(ERROR, "unrecognized join type: %d",
 				 (int) jointype);
 			nestjoinOK = false; /* keep compiler quiet */
 			useallclauses = false;
@@ -348,15 +372,40 @@ match_unsorted_outer(Query *root,
 	}
 
 	/*
-	 * Get the best innerjoin indexpath (if any) for this outer rel. It's
-	 * the same for all outer paths.
+	 * If we need to unique-ify the inner path, we will consider only the
+	 * cheapest inner.
 	 */
-	bestinnerjoin = best_innerjoin(innerrel->innerjoin, outerrel->relids,
-								   jointype);
-
-	foreach(i, outerrel->pathlist)
+	if (jointype == JOIN_UNIQUE_INNER)
 	{
-		Path	   *outerpath = (Path *) lfirst(i);
+		inner_cheapest_total = (Path *)
+			create_unique_path(root, innerrel, inner_cheapest_total);
+		inner_cheapest_startup = inner_cheapest_total;
+		jointype = JOIN_INNER;
+	}
+	else if (nestjoinOK)
+	{
+		/*
+		 * If the cheapest inner path is a join or seqscan, we should
+		 * consider materializing it.  (This is a heuristic: we could
+		 * consider it always, but for inner indexscans it's probably a
+		 * waste of time.)
+		 */
+		if (!(IsA(inner_cheapest_total, IndexPath) ||
+			  IsA(inner_cheapest_total, TidPath)))
+			matpath = (Path *)
+				create_material_path(innerrel, inner_cheapest_total);
+
+		/*
+		 * Get the best innerjoin indexpath (if any) for this outer rel.
+		 * It's the same for all outer paths.
+		 */
+		bestinnerjoin = best_inner_indexscan(root, innerrel,
+											 outerrel->relids, jointype);
+	}
+
+	foreach(l, outerrel->pathlist)
+	{
+		Path	   *outerpath = (Path *) lfirst(l);
 		List	   *merge_pathkeys;
 		List	   *mergeclauses;
 		List	   *innersortkeys;
@@ -366,20 +415,33 @@ match_unsorted_outer(Query *root,
 		int			num_sortkeys;
 		int			sortkeycnt;
 
+		/*
+		 * If we need to unique-ify the outer path, it's pointless to
+		 * consider any but the cheapest outer.
+		 */
+		if (save_jointype == JOIN_UNIQUE_OUTER)
+		{
+			if (outerpath != outerrel->cheapest_total_path)
+				continue;
+			outerpath = (Path *) create_unique_path(root, outerrel, outerpath);
+			jointype = JOIN_INNER;
+		}
+
 		/*
 		 * The result will have this sort order (even if it is implemented
 		 * as a nestloop, and even if some of the mergeclauses are
 		 * implemented by qpquals rather than as true mergeclauses):
 		 */
-		merge_pathkeys = build_join_pathkeys(root, joinrel,
+		merge_pathkeys = build_join_pathkeys(root, joinrel, jointype,
 											 outerpath->pathkeys);
 
 		if (nestjoinOK)
 		{
 			/*
 			 * Always consider a nestloop join with this outer and
-			 * cheapest-total-cost inner.	Consider nestloops using the
-			 * cheapest-startup-cost inner as well, and the best innerjoin
+			 * cheapest-total-cost inner.  When appropriate, also consider
+			 * using the materialized form of the cheapest inner, the
+			 * cheapest-startup-cost inner path, and the best innerjoin
 			 * indexpath.
 			 */
 			add_path(joinrel, (Path *)
@@ -387,17 +449,25 @@ match_unsorted_outer(Query *root,
 										  joinrel,
 										  jointype,
 										  outerpath,
-										  innerrel->cheapest_total_path,
+										  inner_cheapest_total,
 										  restrictlist,
 										  merge_pathkeys));
-			if (innerrel->cheapest_startup_path !=
-				innerrel->cheapest_total_path)
+			if (matpath != NULL)
 				add_path(joinrel, (Path *)
 						 create_nestloop_path(root,
 											  joinrel,
 											  jointype,
 											  outerpath,
-										 innerrel->cheapest_startup_path,
+											  matpath,
+											  restrictlist,
+											  merge_pathkeys));
+			if (inner_cheapest_startup != inner_cheapest_total)
+				add_path(joinrel, (Path *)
+						 create_nestloop_path(root,
+											  joinrel,
+											  jointype,
+											  outerpath,
+											  inner_cheapest_startup,
 											  restrictlist,
 											  merge_pathkeys));
 			if (bestinnerjoin != NULL)
@@ -411,15 +481,32 @@ match_unsorted_outer(Query *root,
 											  merge_pathkeys));
 		}
 
+		/* Can't do anything else if outer path needs to be unique'd */
+		if (save_jointype == JOIN_UNIQUE_OUTER)
+			continue;
+
 		/* Look for useful mergeclauses (if any) */
 		mergeclauses = find_mergeclauses_for_pathkeys(root,
 													  outerpath->pathkeys,
 													  mergeclause_list);
 
-		/* Done with this outer path if no chance for a mergejoin */
+		/*
+		 * Done with this outer path if no chance for a mergejoin.
+		 *
+		 * Special corner case: for "x FULL JOIN y ON true", there will be no
+		 * join clauses at all.  Ordinarily we'd generate a clauseless
+		 * nestloop path, but since mergejoin is our only join type that
+		 * supports FULL JOIN, it's necessary to generate a clauseless
+		 * mergejoin path instead.
+		 */
 		if (mergeclauses == NIL)
-			continue;
-		if (useallclauses && length(mergeclauses) != length(mergeclause_list))
+		{
+			if (jointype == JOIN_FULL)
+				 /* okay to try for mergejoin */ ;
+			else
+				continue;
+		}
+		if (useallclauses && list_length(mergeclauses) != list_length(mergeclause_list))
 			continue;
 
 		/* Compute the required ordering of the inner path */
@@ -431,31 +518,34 @@ match_unsorted_outer(Query *root,
 		 * Generate a mergejoin on the basis of sorting the cheapest
 		 * inner. Since a sort will be needed, only cheapest total cost
 		 * matters.  (But create_mergejoin_path will do the right thing if
-		 * innerrel->cheapest_total_path is already correctly sorted.)
+		 * inner_cheapest_total is already correctly sorted.)
 		 */
 		add_path(joinrel, (Path *)
 				 create_mergejoin_path(root,
 									   joinrel,
 									   jointype,
 									   outerpath,
-									   innerrel->cheapest_total_path,
+									   inner_cheapest_total,
 									   restrictlist,
 									   merge_pathkeys,
 									   mergeclauses,
 									   NIL,
 									   innersortkeys));
 
+		/* Can't do anything else if inner path needs to be unique'd */
+		if (save_jointype == JOIN_UNIQUE_INNER)
+			continue;
+
 		/*
 		 * Look for presorted inner paths that satisfy the innersortkey
 		 * list --- or any truncation thereof, if we are allowed to build
 		 * a mergejoin using a subset of the merge clauses.  Here, we
 		 * consider both cheap startup cost and cheap total cost.  Ignore
-		 * innerrel->cheapest_total_path, since we already made a path
-		 * with it.
+		 * inner_cheapest_total, since we already made a path with it.
 		 */
-		num_sortkeys = length(innersortkeys);
+		num_sortkeys = list_length(innersortkeys);
 		if (num_sortkeys > 1 && !useallclauses)
-			trialsortkeys = listCopy(innersortkeys);	/* need modifiable copy */
+			trialsortkeys = list_copy(innersortkeys);	/* need modifiable copy */
 		else
 			trialsortkeys = innersortkeys;		/* won't really truncate */
 		cheapest_startup_inner = NULL;
@@ -471,12 +561,12 @@ match_unsorted_outer(Query *root,
 			 * 'sortkeycnt' innersortkeys.	NB: trialsortkeys list is
 			 * modified destructively, which is why we made a copy...
 			 */
-			trialsortkeys = ltruncate(sortkeycnt, trialsortkeys);
+			trialsortkeys = list_truncate(trialsortkeys, sortkeycnt);
 			innerpath = get_cheapest_path_for_pathkeys(innerrel->pathlist,
 													   trialsortkeys,
 													   TOTAL_COST);
 			if (innerpath != NULL &&
-				innerpath != innerrel->cheapest_total_path &&
+				innerpath != inner_cheapest_total &&
 				(cheapest_total_inner == NULL ||
 				 compare_path_costs(innerpath, cheapest_total_inner,
 									TOTAL_COST) < 0))
@@ -511,7 +601,7 @@ match_unsorted_outer(Query *root,
 													   trialsortkeys,
 													   STARTUP_COST);
 			if (innerpath != NULL &&
-				innerpath != innerrel->cheapest_total_path &&
+				innerpath != inner_cheapest_total &&
 				(cheapest_startup_inner == NULL ||
 				 compare_path_costs(innerpath, cheapest_startup_inner,
 									STARTUP_COST) < 0))
@@ -560,150 +650,10 @@ match_unsorted_outer(Query *root,
 	}
 }
 
-#ifdef NOT_USED
-
-/*
- * match_unsorted_inner
- *	  Generate mergejoin paths that use an explicit sort of the outer path
- *	  with an already-ordered inner path.
- *
- * 'joinrel' is the join result relation
- * 'outerrel' is the outer join relation
- * 'innerrel' is the inner join relation
- * 'restrictlist' contains all of the RestrictInfo nodes for restriction
- *		clauses that apply to this join
- * 'mergeclause_list' is a list of RestrictInfo nodes for available
- *		mergejoin clauses in this join
- * 'jointype' is the type of join to do
- */
-static void
-match_unsorted_inner(Query *root,
-					 RelOptInfo *joinrel,
-					 RelOptInfo *outerrel,
-					 RelOptInfo *innerrel,
-					 List *restrictlist,
-					 List *mergeclause_list,
-					 JoinType jointype)
-{
-	bool		useallclauses;
-	List	   *i;
-
-	switch (jointype)
-	{
-		case JOIN_INNER:
-		case JOIN_LEFT:
-			useallclauses = false;
-			break;
-		case JOIN_RIGHT:
-		case JOIN_FULL:
-			useallclauses = true;
-			break;
-		default:
-			elog(ERROR, "match_unsorted_inner: unexpected join type %d",
-				 (int) jointype);
-			useallclauses = false;		/* keep compiler quiet */
-			break;
-	}
-
-	foreach(i, innerrel->pathlist)
-	{
-		Path	   *innerpath = (Path *) lfirst(i);
-		List	   *mergeclauses;
-		List	   *outersortkeys;
-		List	   *merge_pathkeys;
-		Path	   *totalouterpath;
-		Path	   *startupouterpath;
-
-		/* Look for useful mergeclauses (if any) */
-		mergeclauses = find_mergeclauses_for_pathkeys(root,
-													  innerpath->pathkeys,
-													  mergeclause_list);
-
-		/* Done with this inner path if no chance for a mergejoin */
-		if (mergeclauses == NIL)
-			continue;
-		if (useallclauses && length(mergeclauses) != length(mergeclause_list))
-			continue;
-
-		/* Compute the required ordering of the outer path */
-		outersortkeys = make_pathkeys_for_mergeclauses(root,
-													   mergeclauses,
-													   outerrel);
-
-		/*
-		 * Generate a mergejoin on the basis of sorting the cheapest
-		 * outer. Since a sort will be needed, only cheapest total cost
-		 * matters.
-		 */
-		merge_pathkeys = build_join_pathkeys(root, joinrel, outersortkeys);
-		add_path(joinrel, (Path *)
-				 create_mergejoin_path(root,
-									   joinrel,
-									   jointype,
-									   outerrel->cheapest_total_path,
-									   innerpath,
-									   restrictlist,
-									   merge_pathkeys,
-									   mergeclauses,
-									   outersortkeys,
-									   NIL));
-
-		/*
-		 * Now generate mergejoins based on already-sufficiently-ordered
-		 * outer paths.  There's likely to be some redundancy here with
-		 * paths already generated by merge_unsorted_outer ... but since
-		 * merge_unsorted_outer doesn't consider all permutations of the
-		 * mergeclause list, it may fail to notice that this particular
-		 * innerpath could have been used with this outerpath.
-		 */
-		totalouterpath = get_cheapest_path_for_pathkeys(outerrel->pathlist,
-														outersortkeys,
-														TOTAL_COST);
-		if (totalouterpath == NULL)
-			continue;			/* there won't be a startup-cost path
-								 * either */
-
-		merge_pathkeys = build_join_pathkeys(root, joinrel,
-											 totalouterpath->pathkeys);
-		add_path(joinrel, (Path *)
-				 create_mergejoin_path(root,
-									   joinrel,
-									   jointype,
-									   totalouterpath,
-									   innerpath,
-									   restrictlist,
-									   merge_pathkeys,
-									   mergeclauses,
-									   NIL,
-									   NIL));
-
-		startupouterpath = get_cheapest_path_for_pathkeys(outerrel->pathlist,
-														  outersortkeys,
-														  STARTUP_COST);
-		if (startupouterpath != NULL && startupouterpath != totalouterpath)
-		{
-			merge_pathkeys = build_join_pathkeys(root, joinrel,
-											 startupouterpath->pathkeys);
-			add_path(joinrel, (Path *)
-					 create_mergejoin_path(root,
-										   joinrel,
-										   jointype,
-										   startupouterpath,
-										   innerpath,
-										   restrictlist,
-										   merge_pathkeys,
-										   mergeclauses,
-										   NIL,
-										   NIL));
-		}
-	}
-}
-#endif
-
 /*
  * hash_inner_and_outer
  *	  Create hashjoin join paths by explicitly hashing both the outer and
- *	  inner join relations of each available hash clause.
+ *	  inner keys of each available hash clause.
  *
  * 'joinrel' is the join relation
  * 'outerrel' is the outer join relation
@@ -720,17 +670,19 @@ hash_inner_and_outer(Query *root,
 					 List *restrictlist,
 					 JoinType jointype)
 {
-	Relids		outerrelids = outerrel->relids;
-	Relids		innerrelids = innerrel->relids;
 	bool		isouterjoin;
-	List	   *i;
+	List	   *hashclauses;
+	ListCell   *l;
 
 	/*
-	 * Hashjoin only supports inner and left joins.
+	 * Hashjoin only supports inner, left, and IN joins.
 	 */
 	switch (jointype)
 	{
 		case JOIN_INNER:
+		case JOIN_IN:
+		case JOIN_UNIQUE_OUTER:
+		case JOIN_UNIQUE_INNER:
 			isouterjoin = false;
 			break;
 		case JOIN_LEFT:
@@ -741,142 +693,94 @@ hash_inner_and_outer(Query *root,
 	}
 
 	/*
+	 * We need to build only one hashpath for any given pair of outer and
+	 * inner relations; all of the hashable clauses will be used as keys.
+	 *
 	 * Scan the join's restrictinfo list to find hashjoinable clauses that
-	 * are usable with this pair of sub-relations.	Since we currently
-	 * accept only var-op-var clauses as hashjoinable, we need only check
-	 * the membership of the vars to determine whether a particular clause
-	 * can be used with this pair of sub-relations.  This code would need
-	 * to be upgraded if we wanted to allow more-complex expressions in
-	 * hash joins.
+	 * are usable with this pair of sub-relations.
 	 */
-	foreach(i, restrictlist)
+	hashclauses = NIL;
+	foreach(l, restrictlist)
 	{
-		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(i);
-		Var		   *left,
-				   *right;
-		List	   *hashclauses;
+		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(l);
 
-		if (restrictinfo->hashjoinoperator == InvalidOid)
+		if (!restrictinfo->can_join ||
+			restrictinfo->hashjoinoperator == InvalidOid)
 			continue;			/* not hashjoinable */
 
 		/*
 		 * If processing an outer join, only use its own join clauses for
 		 * hashing.  For inner joins we need not be so picky.
 		 */
-		if (isouterjoin && restrictinfo->ispusheddown)
+		if (isouterjoin && restrictinfo->is_pushed_down)
 			continue;
 
-		/* these must be OK, since check_hashjoinable accepted the clause */
-		left = get_leftop(restrictinfo->clause);
-		right = get_rightop(restrictinfo->clause);
-
 		/*
 		 * Check if clause is usable with these input rels.
 		 */
-		if (intMember(left->varno, outerrelids) &&
-			intMember(right->varno, innerrelids))
+		if (bms_is_subset(restrictinfo->left_relids, outerrel->relids) &&
+			bms_is_subset(restrictinfo->right_relids, innerrel->relids))
 		{
 			/* righthand side is inner */
 		}
-		else if (intMember(left->varno, innerrelids) &&
-				 intMember(right->varno, outerrelids))
+		else if (bms_is_subset(restrictinfo->left_relids, innerrel->relids) &&
+			 bms_is_subset(restrictinfo->right_relids, outerrel->relids))
 		{
 			/* lefthand side is inner */
 		}
 		else
 			continue;			/* no good for these input relations */
 
-		/* always a one-element list of hash clauses */
-		hashclauses = makeList1(restrictinfo);
+		hashclauses = lappend(hashclauses, restrictinfo);
+	}
 
+	/* If we found any usable hashclauses, make a path */
+	if (hashclauses)
+	{
 		/*
 		 * We consider both the cheapest-total-cost and
 		 * cheapest-startup-cost outer paths.  There's no need to consider
 		 * any but the cheapest-total-cost inner path, however.
 		 */
+		Path	   *cheapest_startup_outer = outerrel->cheapest_startup_path;
+		Path	   *cheapest_total_outer = outerrel->cheapest_total_path;
+		Path	   *cheapest_total_inner = innerrel->cheapest_total_path;
+
+		/* Unique-ify if need be */
+		if (jointype == JOIN_UNIQUE_OUTER)
+		{
+			cheapest_total_outer = (Path *)
+				create_unique_path(root, outerrel, cheapest_total_outer);
+			cheapest_startup_outer = cheapest_total_outer;
+			jointype = JOIN_INNER;
+		}
+		else if (jointype == JOIN_UNIQUE_INNER)
+		{
+			cheapest_total_inner = (Path *)
+				create_unique_path(root, innerrel, cheapest_total_inner);
+			jointype = JOIN_INNER;
+		}
+
 		add_path(joinrel, (Path *)
 				 create_hashjoin_path(root,
 									  joinrel,
 									  jointype,
-									  outerrel->cheapest_total_path,
-									  innerrel->cheapest_total_path,
+									  cheapest_total_outer,
+									  cheapest_total_inner,
 									  restrictlist,
 									  hashclauses));
-		if (outerrel->cheapest_startup_path != outerrel->cheapest_total_path)
+		if (cheapest_startup_outer != cheapest_total_outer)
 			add_path(joinrel, (Path *)
 					 create_hashjoin_path(root,
 										  joinrel,
 										  jointype,
-										  outerrel->cheapest_startup_path,
-										  innerrel->cheapest_total_path,
+										  cheapest_startup_outer,
+										  cheapest_total_inner,
 										  restrictlist,
 										  hashclauses));
 	}
 }
 
-/*
- * best_innerjoin
- *	  Find the cheapest index path that has already been identified by
- *	  indexable_joinclauses() as being a possible inner path for the given
- *	  outer relation(s) in a nestloop join.
- *
- * We compare indexpaths on total_cost only, assuming that they will all have
- * zero or negligible startup_cost.  We might have to think harder someday...
- *
- * 'join_paths' is a list of potential inner indexscan join paths
- * 'outer_relids' is the relid list of the outer join relation
- *
- * Returns the pathnode of the best path, or NULL if there's no
- * usable path.
- */
-static Path *
-best_innerjoin(List *join_paths, Relids outer_relids, JoinType jointype)
-{
-	Path	   *cheapest = (Path *) NULL;
-	bool		isouterjoin;
-	List	   *join_path;
-
-	/*
-	 * Nestloop only supports inner and left joins.
-	 */
-	switch (jointype)
-	{
-		case JOIN_INNER:
-			isouterjoin = false;
-			break;
-		case JOIN_LEFT:
-			isouterjoin = true;
-			break;
-		default:
-			return NULL;
-	}
-
-	foreach(join_path, join_paths)
-	{
-		IndexPath  *path = (IndexPath *) lfirst(join_path);
-
-		Assert(IsA(path, IndexPath));
-
-		/*
-		 * If processing an outer join, only use explicit join clauses in
-		 * the inner indexscan.  For inner joins we need not be so picky.
-		 */
-		if (isouterjoin && !path->alljoinquals)
-			continue;
-
-		/*
-		 * path->joinrelids is the set of base rels that must be part of
-		 * outer_relids in order to use this inner path, because those
-		 * rels are used in the index join quals of this inner path.
-		 */
-		if (is_subseti(path->joinrelids, outer_relids) &&
-			(cheapest == NULL ||
-			 compare_path_costs((Path *) path, cheapest, TOTAL_COST) < 0))
-			cheapest = (Path *) path;
-	}
-	return cheapest;
-}
-
 /*
  * select_mergejoin_clauses
  *	  Select mergejoin clauses that are usable for a particular join.
@@ -885,13 +789,6 @@ best_innerjoin(List *join_paths, Relids outer_relids, JoinType jointype)
  * We examine each restrictinfo clause known for the join to see
  * if it is mergejoinable and involves vars from the two sub-relations
  * currently of interest.
- *
- * Since we currently allow only plain Vars as the left and right sides
- * of mergejoin clauses, this test is relatively simple.  This routine
- * would need to be upgraded to support more-complex expressions
- * as sides of mergejoins.	In theory, we could allow arbitrarily complex
- * expressions in mergejoins, so long as one side uses only vars from one
- * sub-relation and the other side uses only vars from the other.
  */
 static List *
 select_mergejoin_clauses(RelOptInfo *joinrel,
@@ -901,62 +798,76 @@ select_mergejoin_clauses(RelOptInfo *joinrel,
 						 JoinType jointype)
 {
 	List	   *result_list = NIL;
-	Relids		outerrelids = outerrel->relids;
-	Relids		innerrelids = innerrel->relids;
 	bool		isouterjoin = IS_OUTER_JOIN(jointype);
-	List	   *i;
+	bool		have_nonmergeable_joinclause = false;
+	ListCell   *l;
 
-	foreach(i, restrictlist)
+	foreach(l, restrictlist)
 	{
-		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(i);
-		Expr	   *clause;
-		Var		   *left,
-				   *right;
+		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(l);
 
 		/*
-		 * If processing an outer join, only use its own join clauses in
-		 * the merge.  For inner joins we need not be so picky.
-		 *
-		 * Furthermore, if it is a right/full join then *all* the explicit
-		 * join clauses must be mergejoinable, else the executor will
-		 * fail. If we are asked for a right join then just return NIL to
-		 * indicate no mergejoin is possible (we can handle it as a left
-		 * join instead). If we are asked for a full join then emit an
-		 * error, because there is no fallback.
+		 * If processing an outer join, only use its own join clauses in the
+		 * merge.  For inner joins we can use pushed-down clauses too.
+		 * (Note: we don't set have_nonmergeable_joinclause here because
+		 * pushed-down clauses will become otherquals not joinquals.)
 		 */
-		if (isouterjoin)
+		if (isouterjoin && restrictinfo->is_pushed_down)
+			continue;
+
+		if (!restrictinfo->can_join ||
+			restrictinfo->mergejoinoperator == InvalidOid)
 		{
-			if (restrictinfo->ispusheddown)
-				continue;
-			switch (jointype)
-			{
-				case JOIN_RIGHT:
-					if (restrictinfo->mergejoinoperator == InvalidOid)
-						return NIL;		/* not mergejoinable */
-					break;
-				case JOIN_FULL:
-					if (restrictinfo->mergejoinoperator == InvalidOid)
-						elog(ERROR, "FULL JOIN is only supported with mergejoinable join conditions");
-					break;
-				default:
-					/* otherwise, it's OK to have nonmergeable join quals */
-					break;
-			}
+			have_nonmergeable_joinclause = true;
+			continue;			/* not mergejoinable */
 		}
 
-		if (restrictinfo->mergejoinoperator == InvalidOid)
-			continue;			/* not mergejoinable */
+		/*
+		 * Check if clause is usable with these input rels.  All the vars
+		 * needed on each side of the clause must be available from one or
+		 * the other of the input rels.
+		 */
+		if (bms_is_subset(restrictinfo->left_relids, outerrel->relids) &&
+			bms_is_subset(restrictinfo->right_relids, innerrel->relids))
+		{
+			/* righthand side is inner */
+		}
+		else if (bms_is_subset(restrictinfo->left_relids, innerrel->relids) &&
+			 bms_is_subset(restrictinfo->right_relids, outerrel->relids))
+		{
+			/* lefthand side is inner */
+		}
+		else
+		{
+			have_nonmergeable_joinclause = true;
+			continue;			/* no good for these input relations */
+		}
 
-		clause = restrictinfo->clause;
-		/* these must be OK, since check_mergejoinable accepted the clause */
-		left = get_leftop(clause);
-		right = get_rightop(clause);
+		result_list = lcons(restrictinfo, result_list);
+	}
 
-		if ((intMember(left->varno, outerrelids) &&
-			 intMember(right->varno, innerrelids)) ||
-			(intMember(left->varno, innerrelids) &&
-			 intMember(right->varno, outerrelids)))
-			result_list = lcons(restrictinfo, result_list);
+	/*
+	 * If it is a right/full join then *all* the explicit join clauses must be
+	 * mergejoinable, else the executor will fail. If we are asked for a right
+	 * join then just return NIL to indicate no mergejoin is possible (we can
+	 * handle it as a left join instead). If we are asked for a full join then
+	 * emit an error, because there is no fallback.
+	 */
+	if (have_nonmergeable_joinclause)
+	{
+		switch (jointype)
+		{
+			case JOIN_RIGHT:
+				return NIL;		/* not mergejoinable */
+			case JOIN_FULL:
+				ereport(ERROR,
+						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+						 errmsg("FULL JOIN is only supported with merge-joinable join conditions")));
+				break;
+			default:
+				/* otherwise, it's OK to have nonmergeable join quals */
+				break;
+		}
 	}
 
 	return result_list;
