@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2003, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2005, PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql-server/src/bin/psql/print.c,v 1.43 2003/08/14 18:49:42 tgl Exp $
+ * $PostgreSQL: pgsql/src/bin/psql/print.c,v 1.54 2005/01/01 05:43:08 momjian Exp $
  */
 #include "postgres_fe.h"
 #include "common.h"
@@ -12,7 +12,7 @@
 #include <math.h>
 #include <signal.h>
 
-#if !defined(_MSC_VER) && !defined(__BORLANDC__)
+#ifndef WIN32_CLIENT_ONLY
 #include <unistd.h>
 #endif
 
@@ -227,14 +227,14 @@ print_aligned_text(const char *title, const char *const * headers,
 		widths = calloc(col_count, sizeof(*widths));
 		if (!widths)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 
 		head_w = calloc(col_count, sizeof(*head_w));
 		if (!head_w)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -253,7 +253,7 @@ print_aligned_text(const char *title, const char *const * headers,
 		cell_w = calloc(cell_count, sizeof(*cell_w));
 		if (!cell_w)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -388,7 +388,14 @@ print_aligned_text(const char *title, const char *const * headers,
 		for (ptr = footers; *ptr; ptr++)
 			fprintf(fout, "%s\n", *ptr);
 
+#ifndef __MINGW32__
+
+	/*
+	 * for some reason MinGW outputs an extra newline, so this supresses
+	 * it
+	 */
 	fputc('\n', fout);
+#endif
 
 	/* clean up */
 	free(cell_w);
@@ -430,7 +437,7 @@ print_aligned_vertical(const char *title, const char *const * headers,
 		head_w = calloc(col_count, sizeof(*head_w));
 		if (!head_w)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -454,7 +461,7 @@ print_aligned_vertical(const char *title, const char *const * headers,
 		cell_w = calloc(cell_count, sizeof(*cell_w));
 		if (!cell_w)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -478,7 +485,7 @@ print_aligned_vertical(const char *title, const char *const * headers,
 	divider = malloc(hwidth + dwidth + 10);
 	if (!divider)
 	{
-		perror("malloc");
+		fprintf(stderr, gettext("out of memory\n"));
 		exit(EXIT_FAILURE);
 	}
 	divider[0] = '\0';
@@ -507,7 +514,7 @@ print_aligned_vertical(const char *title, const char *const * headers,
 
 				if (!record_str)
 				{
-					perror("malloc");
+					fprintf(stderr, gettext("out of memory\n"));
 					exit(EXIT_FAILURE);
 				}
 
@@ -525,7 +532,7 @@ print_aligned_vertical(const char *title, const char *const * headers,
 
 					if (!div_copy)
 					{
-						perror("malloc");
+						fprintf(stderr, gettext("out of memory\n"));
 						exit(EXIT_FAILURE);
 					}
 
@@ -766,7 +773,7 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 
 
 /*************************/
-/* LaTeX				 */
+/* LaTeX		 */
 /*************************/
 
 
@@ -787,6 +794,9 @@ latex_escaped_print(const char *in, FILE *fout)
 			case '$':
 				fputs("\\$", fout);
 				break;
+			case '_':
+				fputs("\\_", fout);
+				break;
 			case '{':
 				fputs("\\{", fout);
 				break;
@@ -814,7 +824,6 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 {
 	unsigned int col_count = 0;
 	unsigned int i;
-	const char *cp;
 	const char *const * ptr;
 
 
@@ -826,42 +835,39 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 		fputs("\n\\end{center}\n\n", fout);
 	}
 
+	/* count columns */
+	for (ptr = headers; *ptr; ptr++)
+		col_count++;
+
 	/* begin environment and set alignments and borders */
 	fputs("\\begin{tabular}{", fout);
-	if (opt_border == 0)
-		fputs(opt_align, fout);
-	else if (opt_border == 1)
+
+	if (opt_border == 2)
+		fputs("| ", fout);
+	for (i = 0; i < col_count; i++)
 	{
-		for (cp = opt_align; *cp; cp++)
-		{
-			if (cp != opt_align)
-				fputc('|', fout);
-			fputc(*cp, fout);
-		}
-	}
-	else if (opt_border == 2)
-	{
-		for (cp = opt_align; *cp; cp++)
-		{
-			fputc('|', fout);
-			fputc(*cp, fout);
-		}
-		fputc('|', fout);
+		fputc(*(opt_align + i), fout);
+		if (opt_border != 0 && i < col_count - 1)
+			fputs(" | ", fout);
 	}
+	if (opt_border == 2)
+		fputs(" |", fout);
+
 	fputs("}\n", fout);
 
 	if (!opt_barebones && opt_border == 2)
 		fputs("\\hline\n", fout);
 
 	/* print headers and count columns */
-	for (i = 0, ptr = headers; *ptr; i++, ptr++)
+	for (i = 0, ptr = headers; i < col_count; i++, ptr++)
 	{
-		col_count++;
 		if (!opt_barebones)
 		{
 			if (i != 0)
 				fputs(" & ", fout);
+			fputs("\\textit{", fout);
 			latex_escaped_print(*ptr, fout);
+			fputc('}', fout);
 		}
 	}
 
@@ -885,7 +891,7 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 	if (opt_border == 2)
 		fputs("\\hline\n", fout);
 
-	fputs("\\end{tabular}\n\n", fout);
+	fputs("\\end{tabular}\n\n\\noindent ", fout);
 
 
 	/* print footers */
@@ -948,8 +954,12 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 			if (!opt_barebones)
 			{
 				if (opt_border == 2)
+				{
 					fputs("\\hline\n", fout);
-				fprintf(fout, "\\multicolumn{2}{c}{Record %d} \\\\\n", record++);
+					fprintf(fout, "\\multicolumn{2}{|c|}{\\textit{Record %d}} \\\\\n", record++);
+				}
+				else
+					fprintf(fout, "\\multicolumn{2}{c}{\\textit{Record %d}} \\\\\n", record++);
 			}
 			if (opt_border >= 1)
 				fputs("\\hline\n", fout);
@@ -964,7 +974,7 @@ const char *opt_align, bool opt_barebones, unsigned short int opt_border,
 	if (opt_border == 2)
 		fputs("\\hline\n", fout);
 
-	fputs("\\end{tabular}\n\n", fout);
+	fputs("\\end{tabular}\n\n\\noindent ", fout);
 
 
 	/* print footers */
@@ -1130,21 +1140,20 @@ void
 printQuery(const PGresult *result, const printQueryOpt *opt, FILE *fout)
 {
 	int			nfields;
+	int			ncells;
 	const char **headers;
 	const char **cells;
 	char	  **footers;
 	char	   *align;
 	int			i;
 
-
 	/* extract headers */
-
 	nfields = PQnfields(result);
 
 	headers = calloc(nfields + 1, sizeof(*headers));
 	if (!headers)
 	{
-		perror("calloc");
+		fprintf(stderr, gettext("out of memory\n"));
 		exit(EXIT_FAILURE);
 	}
 
@@ -1152,15 +1161,15 @@ printQuery(const PGresult *result, const printQueryOpt *opt, FILE *fout)
 		headers[i] = mbvalidate(PQfname(result, i), opt->topt.encoding);
 
 	/* set cells */
-
-	cells = calloc(nfields * PQntuples(result) + 1, sizeof(*cells));
+	ncells = PQntuples(result) * nfields;
+	cells = calloc(ncells + 1, sizeof(*cells));
 	if (!cells)
 	{
-		perror("calloc");
+		fprintf(stderr, gettext("out of memory\n"));
 		exit(EXIT_FAILURE);
 	}
 
-	for (i = 0; i < nfields * PQntuples(result); i++)
+	for (i = 0; i < ncells; i++)
 	{
 		if (PQgetisnull(result, i / nfields, i % nfields))
 			cells[i] = opt->nullPrint ? opt->nullPrint : "";
@@ -1177,11 +1186,16 @@ printQuery(const PGresult *result, const printQueryOpt *opt, FILE *fout)
 		footers = calloc(2, sizeof(*footers));
 		if (!footers)
 		{
-			perror("calloc");
+			fprintf(stderr, gettext("out of memory\n"));
 			exit(EXIT_FAILURE);
 		}
 
 		footers[0] = malloc(100);
+		if (!footers[0])
+		{
+			fprintf(stderr, gettext("out of memory\n"));
+			exit(EXIT_FAILURE);
+		}
 		if (PQntuples(result) == 1)
 			snprintf(footers[0], 100, gettext("(1 row)"));
 		else
@@ -1191,11 +1205,10 @@ printQuery(const PGresult *result, const printQueryOpt *opt, FILE *fout)
 		footers = NULL;
 
 	/* set alignment */
-
 	align = calloc(nfields + 1, sizeof(*align));
 	if (!align)
 	{
-		perror("calloc");
+		fprintf(stderr, gettext("out of memory\n"));
 		exit(EXIT_FAILURE);
 	}
 
@@ -1218,13 +1231,12 @@ printQuery(const PGresult *result, const printQueryOpt *opt, FILE *fout)
 	}
 
 	/* call table printer */
-
 	printTable(opt->title, headers, cells,
-			   footers ? (const char *const *) footers : (const char *const *) (opt->footers),
+			   (const char *const *) footers,
 			   align, &opt->topt, fout);
 
-	free((void *) headers);
-	free((void *) cells);
+	free(headers);
+	free(cells);
 	if (footers)
 	{
 		free(footers[0]);
