@@ -2,10 +2,10 @@
  *
  * dropdb
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Header: /cvsroot/pgsql-server/src/bin/scripts/dropdb.c,v 1.6.4.1 2004/01/01 19:27:28 tgl Exp $
+ * $PostgreSQL: pgsql/src/bin/scripts/dropdb.c,v 1.14 2004/12/31 22:03:17 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -32,7 +32,7 @@ main(int argc, char *argv[])
 		{NULL, 0, NULL, 0}
 	};
 
-	char	   *progname;
+	const char *progname;
 	int			optindex;
 	int			c;
 
@@ -51,7 +51,8 @@ main(int argc, char *argv[])
 	PGresult   *result;
 
 	progname = get_progname(argv[0]);
-	init_nls();
+	set_pglocale_pgservice(argv[0], "pgscripts");
+
 	handle_help_version_opts(argc, argv, "dropdb", help);
 
 	while ((c = getopt_long(argc, argv, "h:p:U:Weqi", long_options, &optindex)) != -1)
